
library(gplots)
library(xtable)

do.boolean <- TRUE # analyze boolean or not?

only.parameterized <- TRUE

if(do.boolean) {
	out.dir <- "plots-Boolean/"
} else {
	out.dir <- "plots-All/";
}

options(width=Sys.getenv("COLUMNS"))

source("misc-lib.R")
source("sort.R")
source("HSFL-Shared.R")

# in.hyp.type <- "RAW"
model.dir <- "../Model-Vicare/run/"

# ## Read in the subject data 
if( ! is.element("d.ag", ls())) { # (only if we haven't already)
	d.ag <- load.ag.subject.data()
}

# Get all 
model.directories <- list.files(model.dir)
model.directories <- model.directories[ (do.boolean == grepl("BOOLEAN", model.directories)) 
					& (grepl("parameterized", model.directories) | (!only.parameterized) | grepl("_NA_", model.directories) ) ]

################################################################################################################################
################################################################################################################################
#### Main data loop
################################################################################################################################
################################################################################################################################

if(! file.exists(paste(out.dir,"/modelVhuman",sep=""))) {
	dir.create(paste(out.dir,"/modelVhuman",sep=""))
}
D <- NULL
conceptwise.ll <- NULL # each row is a concept, column is a RL

for(md in model.directories) {
	
	print(paste(model.dir, md, "/across-samples.txt", sep=""))
	mparam <- params2hash(md)

	# fix this for the non-LOT models, so we can just call them "language"
	if(mparam["LANGUAGE"] == "NA") { mparam["LANGUAGE"] <- mparam["MODEL"] }

	## TODO: If we want these, go back and get them!
# 	# do a plot of this model's R^2s
# 	bitmap( paste("modelplots/",md,"-Rsq.png",sep=""), height=2,width=4, res=200)
# 	h <- hist(rsqs, breaks=seq(-1,1,0.1), freq=F, col=1, main="", xlab="Correlation", ylab="",cex.axis=2, cex.lab=2, font=2)
# 	abline(v=0)
# 	dev.off()
# 	
	################################################################
	## Compute the model stats
	x <- compute.model.stats(paste(model.dir, md, sep=""), d.ag)
	newD <- x[[2]]
	newD$md <- md # save the model directory here
	D <- rbind(D, newD )  #this sets last.ho.ll globally, which we copy
	
	if(is.null(conceptwise.ll) ) {
		conceptwise.ll <-  data.frame(xxx=x[[3]])
		names(conceptwise.ll) <- md
	} else {
		df <- data.frame(md=x[[3]])
		names(df) <- md
		conceptwise.ll <- cbind(conceptwise.ll, df) # copy the globally set variable ###NOTE: To get this to work, you may have to 
	}

	################################################################
	## Now print the model predicted/real correlation plots
# 
	xes <- NULL
	yes <- NULL	
	xes.ho <- NULL
	yes.ho <- NULL

	for(pf in list.files(paste(model.dir, md, "/predictive", sep=""))) {
		
		d.pf <- read.table(paste(model.dir, md, "/predictive/", pf, sep=""), header=F)
		param.pf <- params2hash(pf)
		pred <- t( d.pf[1,-c(1,2)])
		q <- d.ag[d.ag$concept == param.pf["CONCEPT"] & d.ag$list == param.pf["LIST"],]
		q <- merge(q, ho.key) # merge in the held out key
		q <- sort(q, by= ~ set.number + response.number)
		
		if(length(pred) == length(q$p)) {
			xes <- c( xes, pred )
			yes <- c( yes, q$p )
			
			xes.ho <- c(xes.ho, pred[q$is.held.out])
			yes.ho <- c(yes.ho, q$p[q$is.held.out])
		}
		else {
			print("Mismatch in length:")
			print(pf)
		}

# 		mVacc <- rbind(mVacc, 
	}
	
	a <- aggregate2( yes, by=list( floor(xes * 25)/25 ) )
	a.ho <- aggregate2( yes.ho, by=list( floor(xes.ho * 25)/25 ) )
	
	# plot the model vs. human 
	postscript(paste(out.dir, "/modelVhuman/LANGUAGE_", mparam["LANGUAGE"], "__MODEL_", mparam["MODEL"], ".eps", sep=""), height=5, width=5)
# 	bitmap(paste(out.dir, "/modelVhuman/LANGUAGE_", mparam["LANGUAGE"], "__MODEL_", mparam["MODEL"], ".png", sep=""), height=2.25, width=2.25, res=500, gaa=2, taa=2)
	cex <- 1.2
	plot(NULL, NULL, xlim=c(0,1), ylim=c(0,1), pch=20, cex=cex, col="#CCCCCC", main="", xlab="Model predictions", ylab="Human responses")
	h <- hist2d(xes, yes, nbins=15, show=F)
	image(log(h$counts+1), zlim=c(0, 10), add=T, col=grey.colors(100, start=1.0, end=0.3))
	points( a$X, a$F1, pch=16, cex=cex)
	errbar( a$X, a$F1, a$F1+a$F2, a$F1-a$F2, add=T, pch=".")
	par(col=4)
	points( a.ho$X, a.ho$F1, pch=16, cex=cex, col=4)
	errbar( a.ho$X, a.ho$F1, a.ho$F1+a.ho$F2, a.ho$F1-a.ho$F2, add=T, pch=".")
	par(col=1)
	abline(0,1, lwd=1.5)
	dev.off()
	


}


conceptwise.ll <- conceptwise.ll[conceptwise.ll[,1]<0,] # throw out the zero rows (if any is zero, they all should be)


################################################################
## Now make a comparison table

saveD <- D

# And sort it
D <- sort(D, by= ~ -HO.LL)

## Now let's do some elementwise comparisons
pv <- NULL
pvD <- D # make a copy since we modify below
for(i in 2:10) {
	w <- wilcox.test(conceptwise.ll[,pvD$md[1]], conceptwise.ll[,pvD$md[i]], paired=TRUE, conf.int=TRUE)
	b <- binom.test( table(conceptwise.ll[,pvD$md[1]] < conceptwise.ll[,pvD$md[i]]))
	tt <- t.test(conceptwise.ll[,pvD$md[1]], conceptwise.ll[,pvD$md[i]], paired=TRUE)
	pv <- rbind(pv, data.frame(language=pvD$LANGUAGE[i],  
				binom.estimate=b$estimate, binom.p.value=b$p.value, 
				wilcox.estimate=w$estimate, wilcox.p.value=w$p.value,
				tt.estimate=tt$estimate, tt.p.value=tt$p.value))
}

## Now compare the booleans on the whole data
if( ! do.boolean ){
	pvb <- NULL
	pvDb <- pvD[- grep("LAM", pvD$md),]
	for(i in 2:nrow(pvDb)) {
		w <- wilcox.test(conceptwise.ll[,pvDb$md[1]], conceptwise.ll[,pvDb$md[i]], paired=TRUE, conf.int=TRUE)
		b <- binom.test( table(conceptwise.ll[,pvDb$md[1]] < conceptwise.ll[,pvDb$md[i]]))
		tt <- t.test(conceptwise.ll[,pvDb$md[1]], conceptwise.ll[,pvDb$md[i]], paired=TRUE)
		pvb <- rbind(pvb, data.frame(language=pvDb$LANGUAGE[i],  
					binom.estimate=b$estimate, binom.p.value=b$p.value, 
					wilcox.estimate=w$estimate, wilcox.p.value=w$p.value,
					tt.estimate=tt$estimate, tt.p.value=tt$p.value))
	}
}





# make the language the model column for other comparisons!
D$LANGUAGE <- ifelse( D$LANGUAGE=="NA", as.character(D$MODEL), as.character(D$LANGUAGE))

D <- D[,c("LANGUAGE", "HO.LL", "BIC",  "free.params", "correlation.mean", "across.concept.correlation")]

# D$MODEL <- paste("\\model{", D$MODEL, "}", sep="")
D$LANGUAGE <- paste("\\grammar{", D$LANGUAGE, "}", sep="")
D$LANGUAGE <- gsub("Language-", "", D$LANGUAGE, perl=T) ## TODO: CHECK THIS


row.names(D) <- NULL

names(D) <- c("Grammar", "H.O. LL", "BIC",  "FP", "$R_{response}^2$", "$R_{mean}^2$")

if(do.boolean) {
print(xtable(D, digits=2, caption="Model comparison results on all languages for Boolean target concepts only. H.O.LL gives the held-out likelihood on data independent from that used to fit the parameters. FP gives the number of free parameters, dominated by the number of rules in the grammar (e.g. one for each primitive). $R_{response}^2$ gives the correlation of proportion correct on raw responses between each model and humans. $R_{mean}^2$ gives the correlation between model and humans on overall concept accuracy.", label="fig:BooleanModelComparison"), 
		file=paste(out.dir, "ModelComparisonTable.tex",sep=""), sanitize.text.function=function(x){x}, include.rownames=F)
} else {

	D2 <- D;

	n <- D$Grammar
	D2 <- D2[,-1] # drop column

	# Here we do our fanciness to make cute columns
	D2 <- cbind( ifelse(grepl("Q2O", n), "$\\checkmark$", "$\\cdot$"), D2)
	D2 <- cbind( ifelse(grepl("SMC", n), "$\\checkmark$", "$\\cdot$"), D2)
	D2 <- cbind( ifelse(grepl("EOQ", n), "$\\checkmark$", "$\\cdot$"), D2)
	# This adds QEF or just the boolean name if there is no LAM (note: This should be changed if we include non-LAMs)
	D2 <- cbind(ifelse(! grepl("LAM", n), n, ifelse(grepl("QEF", n), "$\\checkmark$", "$\\cdot$")), D2)
	
	names(D2) <-  c( #"\\begin{turn}{90} Lambda \\end{turn}", 
			"\\begin{turn}{90} FOL\\end{turn}", 
			"\\begin{turn}{90} One-Or-Fewer \\end{turn}", 
			"\\begin{turn}{90} Small-Cardinalities \\end{turn}", 
			"\\begin{turn}{90} $2$nd-Ord.-Quan. \\end{turn}", 
			"H.O. LL", "BIC",  "FP", "$R_{response}^2$", "$R_{mean}^2$")

print(xtable(D2, digits=2, caption="Model comparison results on all languages with quantifiers. H.O.LL gives the held-out likelihood on data independent from that used to fit the parameters. FP gives the number of free parameters, dominated by the number of rules in the grammar (e.g. one for each primitive). $R_{response}^2$ gives the correlation of proportion correct on raw responses between each model and humans. $R_{mean}^2$ gives the correlation between model and humans on overall concept accuracy.", label="fig:AllModelComparison"), 
		file=paste(out.dir, "ModelComparisonTable-checkmarks.tex",sep=""), sanitize.text.function=function(x){x}, include.rownames=F, align="ccccccccccc")


}


print(D)


