# Analyze the posterior distribution on each model parameters. 

library(gplots)
library(xtable)

do.boolean <- TRUE
only.parameterized <- TRUE
remove.CRD <- FALSE # remove languages with CRD operations

options(width=Sys.getenv("COLUMNS"))

source("misc-lib.R")
source("sort.R")
source("HSFL-Shared.R")

# in.hyp.type <- "RAW"
model.dir <- "../Model-Vicare/run/"

# Get all 
model.directories <- list.files(model.dir)
model.directories <- model.directories[ (do.boolean == grepl("BOOLEAN", model.directories)) 
					& (grepl("parameterized", model.directories) | (!only.parameterized) | grepl("_NA_", model.directories) ) ]
model.directories <- model.directories[! grepl("CRD", model.directories) ]

################################################################################################################################
################################################################################################################################
#### Main data loop
################################################################################################################################
################################################################################################################################

if(remove.CRD) { model.directories <- model.directories[ ! grepl("CRD", model.directories)] }

D <- NULL
for(md in model.directories) {
	
	print(paste(model.dir, md, "/across-samples.txt", sep=""))
	mparam <- params2hash(md)

	# fix this for the non-LOT models, so we can just call them "language"
	if(mparam["LANGUAGE"] == "NA") { mparam["LANGUAGE"] <- mparam["MODEL"] }
    
    d <- read.table(paste(model.dir, md, "/across-samples.txt", sep=""))
    
	names(d) <- c("posterior", "npoints", "FP", "Temperature", "Alpha", "Gamma", "Beta-positive", "Beta-negative")
	
	# toss the first half
	d <- d[(nrow(d)/2):nrow(d),]
	
	d$language <- mparam["LANGUAGE"]
	
    D <- rbind(D, d)
}


library(reshape2)
library(ggplot2)
library(plyr)

# melt into the form we want
M <- melt(D, id.vars=c("language", "npoints", "FP"))

myf <- function(dd){
    q <- quantile(dd$value, probs=c(0.025, 0.5, 0.975))
    data.frame(ymin=q[1], y=q[2], ymax=q[3])
    }

# And apply for each
m <- ddply( M, c("language", "npoints", "FP", "variable"), myf)

# Fix these names and variables
m$language <- gsub("Language-", "", m$language)

#Fix our name coding
m$language <- gsub("LAM-", "", m$language)
m$language <- gsub("QEF", "FOL", m$language)
m$language <- gsub("EOQ", "OOF", m$language)
# SMC are okay
m$language <- gsub("Q20", "2OQ", m$language)

m <- subset(m, language != "exemplar")

plt <- ggplot(subset(m, variable != "posterior"), aes(x=language, y=y, ymin=ymin, ymax=ymax)) + 
    geom_point() + geom_errorbar() + 
    facet_wrap(~variable, scales="free") + 
    xlab("") + ylab("") +
    theme(axis.text.x = element_text(angle = 90, hjust = 1)) 

ggsave("parameter-posteriors-Boolean.eps", plt, height=8, width=10)

    

