##NOTES: 
## - What is going on? Why is human entropy > model? -- They are based on different amounts of data!
## -- it looks like things people do worse, we can predict less well. This suggests that we want a better exploration model: when we don't know the right answer, we aren't modeling people's performance very well. 

source("HSFL-Shared.R")

options(width=Sys.getenv("COLUMNS"))

source("misc-lib.R")
source("sort.R")
source("HSFL-Shared.R")

model.root <- "../Model-Vicare/run/"

# ## Read in the data
if( ! is.element("d.ag", ls())) {
	d.ag <- load.ag.subject.data()
}

# 
################################################################################################################
## This function plots a model and human data in filename--this standardizes our model/human plotting
# for everything in model.dirs, add its plot to c l
plot.curves <- function( filename, c, l, model.dirs=NULL, model.cols=2:8, plot.cor=TRUE, sort=FALSE, legend=FALSE, legend.names=NULL) {

 	postscript(paste(filename, ".eps", sep=""), height=3, width=6)
# 	bitmap(paste(filename, ".png", sep=""), height=1.5, width=3, res=400, taa=2, gaa=2)
	par(mar=c(4,4,1,1))
	
	q <- d.ag[as.character(d.ag$concept)==c & as.character(d.ag$list) == l,]
 	q <- sort(q, by= ~ set.number + response.number + right.answer)

# 	# compute the right answers
# 	z <- d[d$concept==c & d$list==l,]
# 	dra <- aggregate(z$right.answer, by=list( z$response.number, z$set.number), function(x){x[1]})
# 	names(dra) <- c("response.number", "set.number", "right.answer")

	o <- 1:nrow(q)
	if(sort) { o <- order(q$p.correct, decreasing=T) } # plot these ordered. Wow that's ugly

	# plot the human data
	plot(q$p.correct[o], xlim=c(0,nrow(q)), ylim=c(0,1), lwd=2.0, type="l", xlab="Response number", ylab="Accuracy", main=concept.desc[c], font=2)

	# add base rate accuracy
	br <- mean(q$right.answer)
	abline(h=br*br + (1-br)*(1-br), lty=2, col=4)

	plot.cor.offset <- 0.2

	if(! is.null(model.dirs)) {
	for(mdi in 1:length(model.dirs)) {
		md <- model.dirs[mdi]
	 	filename <- paste(model.root, md, "/predictive/predictive__CONCEPT_", c, "__LIST_", l, ".txt", sep="")
	 	m <- read.table(filename)
	 	q$m <- t(m[1,c(-1,-2)]) # merge in the model predictions (dra is sorted the right way)
	 	q$macc <- ifelse(q$right.answer, q$m, 1.0-q$m)	

	 	lines(q$macc[o], col=model.cols[mdi], lty=1, lwd=2) #plot in the right order, whatever it is

		if(plot.cor) { # if we should include the correlation coefficient in the lower right, do it here

			
			pred <- t( m[1,-c(1,2)])
		
			rsq <- Rsq(pred, q$p) #Rsq(q$p.correct, q$macc)

	 		text(nrow(q)-3, plot.cor.offset, format(rsq,digits=2, nsmall=2), font=2, col=model.cols[mdi])
			plot.cor.offset <- plot.cor.offset - 0.08
		}
	}
	}
	
	if(legend) {
		lc <- c(1)
		if(!is.null(model.dirs)) { lc <- c(model.cols[1:length(model.dirs)], 1) }
		legend("bottom", c(legend.names, "Human"), lwd=2, col=lc, cex=1, bg="white")
	}

	dev.off()

}


################################################################################################################
## Now actually plot!


# What if we just plot Simple-Boolean-Additional, DNF, and NAND?

# boolean.to.plot <- c("CONCEPT_BOOLEAN__LANGUAGE_Language-Biconditional__INHYT_RAW__MODEL_parameterized-RR-noBeta",
# #  			"CONCEPT_BOOLEAN__LANGUAGE_Language-CNF__INHYT_RAW__MODEL_parameterized-RR-noBeta",
# #  			"CONCEPT_BOOLEAN__LANGUAGE_Language-NAND__INHYT_RAW__MODEL_parameterized-RR-noBeta",
# # 			"CONCEPT_BOOLEAN__LANGUAGE_NA__MODEL_exemplar"
# 			)
#        

# run.concepts <- concepts[! is.na(concept.class[as.character(concepts)])]

# 
dir.create("plots-Boolean/learning-curves/")
bool.concepts <- concepts[! is.na(concept.class[as.character(concepts)]) & concept.class[as.character(concepts)] == "B"]
for( c in bool.concepts) {
	plot.curves(paste("plots-Boolean/learning-curves/LC-BOOLEAN-",c,sep=""), c, "L2", c("CONCEPT_BOOLEAN__LANGUAGE_Language-FullBoolean__INHYT_RAW__MODEL_parameterized-RR-noBeta"
	  #"CONCEPT_BOOLEAN__LANGUAGE_Language-NAND__INHYT_RAW__MODEL_parameterized-RR-noBeta"
	),
		model.cols=rainbow(3), 
		plot.cor=T,
		legend=T, #(c=="hg13"),
		legend.names=c("FullBoolean grammar")
			)
}


dir.create("plots-All/learning-curves/")
all.concepts <- concepts[! is.na(concept.class[as.character(concepts)])]
for( c in all.concepts) {
	plot.curves(paste("plots-All/learning-curves/LC-ALL-",c,sep=""), c, "L2", 
		c("CONCEPT_ALL__LANGUAGE_Language-FullBoolean__INHYT_RAW__MODEL_parameterized-RR-noBeta",
			"CONCEPT_ALL__LANGUAGE_Language-LAM-QEF-EOQ__INHYT_RAW__MODEL_parameterized-RR-noBeta"
		),

		model.cols=rainbow(3), 
		plot.cor=T,
		legend=(c=="hg63"),
		legend.names=c("FullBoolean", "Best Quantifier")
			)
}
# 				     "../Model-Vicare/run/ALL_LANGUAGE_Language-SimpleBoolean__INHYT_RAW___MODEL_parameterized-RR"))

# 

