// some C code for quickly executing mcmc over the parameters in this model. 
// this will efficiently compute likelihoods, etc,
// and do gradient methods on parameters


#include <stdlib.h>
#include <math.h>
#include <stdio.h>
double neginf = -1.0 * __builtin_inf();

// CAN MAKE THISF ASTER BY PRECOMPUTING THE DECAY TERMS AND THEN SAVING RUNNING_LL IN AN ARRAY BEFORE SUMMING
// SO WE ONLY COMPUTE EACH LOG ONCE. 
#define MAX_SET_SIZE 25
double positive_decay_cache[MAX_SET_SIZE+2];
double negative_decay_cache[MAX_SET_SIZE+2];
double pointwise_ll[MAX_SET_SIZE*100];
double last_positive = -99e99;
double last_negative = -99e99;

// so we can call this from scheme
void c_free(void* ptr) { free(ptr); };

double* c_malloc_double(int n) { 
	return (double*)malloc(n * sizeof(double));
}



// here is a funciton for computing the running ll
// with memory decay. Tests show this is 10X faster with memory decay, but teh analogous no-decay version is 2X slower than scheme code
double* running_ll_Cexternal( 	double alpha,
			double baserate, // the baseline response
			double llBeta_positive,
			double llBeta_negative,
		        int N,
			int* response,
			int* correct,
			int* which_item) {
	
	double* running = (double*)malloc(N * sizeof(double));;
	int last_item = which_item[0];

	// First, compute the distance decay cache so we don't have to keep re-doing this
	if( llBeta_positive != last_positive) { // store (x+1)^beta
		for(int i=1;i<MAX_SET_SIZE+1;i++) positive_decay_cache[i] = pow( (float)(i+1), llBeta_positive);
	}
	if( llBeta_negative != last_negative ) {
		for(int i=1;i<MAX_SET_SIZE+1;i++) negative_decay_cache[i] = pow( (float)(i+1), llBeta_negative);
	}
	
	// now fill up the pointwise LL
	for(int j=1;j<N;j++) {
		double inner = (1.0-alpha)*baserate*response[j] + (1.0-alpha)*(1.0-baserate)*(1.0-response[j]);
		if(correct[j] == response[j]) { pointwise_ll[j] = log(alpha + inner); }
		else			      { pointwise_ll[j] = log(inner); }
	}
	
	// the first thing that is not the first item. 
	int start = 1; 
	while(which_item[start] == which_item[0]) { running[start] = 0; start++; }
	
	double s = 0.0;
	for(int i=start;i<N;i++) {		

		running[i] = 0;
		
		//printf("%f\t%f\t%f\t%f\n", alpha, baserate, llBeta_positive, llBeta_negative);

		for(int j=0;(j<i) && (which_item[j] != which_item[i]);j++) {
			//printf("%d\t%d\t%d\n", response[j], correct[j], which_item[j]);
			
			if(correct[j]) { // if it was a positive vs. negative example
				running[i] += pointwise_ll[j] * positive_decay_cache[ which_item[i]-which_item[j] ]; // look up (x+1)^beta 
			} else {
				running[i] += pointwise_ll[j] * negative_decay_cache[ which_item[i]-which_item[j] ];
			}
		}

		//printf("%d\t%f\t%d\t%d\t%d\n", i, running[i], which_item[i], correct[i], response[i]);

		// and go forward as much as you can!
		while( (i+1) < N && which_item[i+1] == which_item[i]) {
			running[i+1] = running[i]; i++;
		}
		
	}
	
 	return running;
}


// with memory decay. Tests show this is 10X faster with memory decay, but teh analogous no-decay version is 2X slower than scheme code
double* running_ll_Cexternal_OLD( 	double alpha,
			double baserate, // the baseline response
			double llBeta_positive,
			double llBeta_negative,
		        int N,
			int* response,
			int* correct,
			int* which_item) {
	
	double* running = (double*)malloc(N * sizeof(double));;
	int last_item = which_item[0];


	// the first thing that is not the first item. 
	int start = 1; while(which_item[start] == which_item[0]) { running[start] = 0; start++; }
	
	double s = 0.0;
	for(int i=start;i<N;i++) {		

		running[i] = 0;
		
		//printf("%f\t%f\t%f\t%f\n", alpha, baserate, llBeta_positive, llBeta_negative);

		for(int j=0;(j<i) && (which_item[j] != which_item[i]);j++) {
			//printf("%d\t%d\t%d\n", response[j], correct[j], which_item[j]);
			
			// get the ll on this point
			double ll = 0.0;

			if(correct[j] == response[j]) { 
				ll += log(alpha + (1.0-alpha)*baserate*response[j] + (1.0-alpha)*(1.0-baserate)*(1.0-response[j]) ); 
			}
			else { 
				ll += log( (1.0-alpha)*baserate*response[j] + (1.0-alpha)*(1.0-baserate)*(1.0-response[j]) );
			}
			
			if(correct[j]) { // if it was a positive vs. negative example
				running[i] += ll * pow( 1+which_item[i]-which_item[j], llBeta_positive);
			} else {
				running[i] += ll * pow( 1+which_item[i]-which_item[j], llBeta_negative);		
			}
		}

		//printf("%d\t%f\t%d\t%d\t%d\n", i, running[i], which_item[i], correct[i], response[i]);

		// and go forward as much as you can!
		while( (i+1) < N && which_item[i+1] == which_item[i]) {
			running[i+1] = running[i]; i++;
		}
		
	}
	
 	return running;
}


double lse_lazy(double* x, int N){

	double max = -9e99;
	for(int i=0;i<N;i++) 
		if(x[i] > max) max = x[i]; 
	
	double ls = 0.0;
	for(int i=0;i<N;i++) {
		ls += exp(x[i] - max);
	}
	
	return ( log(ls) + max);	
}

