;; This maps every concept description (e.g. "hg04") to a reference function for that concept. 

(define concepts (make-hashtable  (lambda (x) (abs (string-hash x)))
                                  string=?))


; True
(hashtable-set! concepts "hg01" '(lambda (x) (truethunk)) )

; False
(hashtable-set! concepts "hg02" '(lambda (x) (falsethunk)) )

; Blue
(hashtable-set! concepts "hg03" '(lambda (x) (eqv* (color* x) 'blue)) )

; Circles
(hashtable-set! concepts "hg04" '(lambda (x) (eqv* (shape* x) 'circle)) )

; Not circles
(hashtable-set! concepts "hg05" '(lambda (x) (not* (eqv* (shape* x) 'circle)) ))

; Circles or blue
(hashtable-set! concepts "hg06" '(lambda (x) (or* 
                                              (eqv* (shape* x) 'circle)
                                              (eqv* (color* x) 'blue))) )
; Circles or triangles
(hashtable-set! concepts "hg07" '(lambda (x) (or* 
                                              (eqv* (shape* x) 'circle)
                                              (eqv* (shape* x) 'triangle))) )
;; blue or green
(hashtable-set! concepts "hg08" '(lambda (x) (or* 
                                              (eqv* (color* x) 'blue)
                                              (eqv* (color* x) 'green))) )

; Circles and blue
(hashtable-set! concepts "hg09" '(lambda (x) (and* 
                                              (eqv* (shape* x) 'circle)
                                              (eqv* (color* x) 'blue))) ) 


; Circles and [not blue]
(hashtable-set! concepts "hg10" '(lambda (x) (and* 
                                              (eqv* (shape* x) 'circle)
                                              (not (eqv* (color* x) 'blue))) ))

;Not [circles and blue]
(hashtable-set! concepts "hg11" '(lambda (x) (not* (and* 
                                                    (eqv* (shape* x) 'circle)
                                                    (eqv* (color* x) 'blue))) ))          

;Not [circles or blue]
(hashtable-set! concepts "hg12" '(lambda (x) (not* (or* 
                                                    (eqv* (shape* x) 'circle)
                                                    (eqv* (color* x) 'blue))) ))          

;Not[ [Not circles] or blue]
(hashtable-set! concepts "hg13" '(lambda (x) (not* (or* 
                                                    (not* (eqv* (shape* x) 'circle))
                                                    (eqv* (color* x) 'blue))) ))          


; circles XOR blue
(hashtable-set! concepts "hg14" '(lambda (x) (or* 
                                              (and* (eqv* (shape* x) 'circle)
                                                    (not* (eqv* (color* x) 'blue)))
                                              (and* (not* (eqv* (shape* x) 'circle))
                                                    (eqv* (color* x) 'blue)))))

; not [circles XOR blue]
(hashtable-set! concepts "hg15" '(lambda (x) (not* (or* 
                                                    (and* (eqv* (shape* x) 'circle)
                                                          (not* (eqv* (color* x) 'blue)))
                                                    (and* (not* (eqv* (shape* x) 'circle))
                                                          (eqv* (color* x) 'blue))))))

;; circles XOR [not blue]
(hashtable-set! concepts "hg16" '(lambda (x) (or* 
                                              (and* (eqv* (shape* x) 'circle)
                                                    (eqv* (color* x) 'blue))
                                              (and* (not* (eqv* (shape* x) 'circle))
                                                    (not* (eqv* (color* x) 'blue))))))

; Everything iff there is a triangle
(hashtable-set! concepts "hg17" '(lambda (S) (lambda (x) (exists* (lambda (z) (eqv* (shape* z) 'triangle))
                                                                  S))))

; Size=3
(hashtable-set! concepts "hg18" '(lambda (x) (eqv* (size* x) 3)) )

; Size=2
(hashtable-set! concepts "hg19" '(lambda (x) (eqv* (size* x) 2)) )

; Size=1
(hashtable-set! concepts "hg20" '(lambda (x) (eqv* (size* x) 1)) )


; Size=3 or 1
(hashtable-set! concepts "hg21" '(lambda (x) (or* (eqv* (size* x) 3)
                                                  (eqv* (size* x) 1))))

; Size= 3 or 2
(hashtable-set! concepts "hg22" '(lambda (x) (or* (eqv* (size* x) 3)
                                                  (eqv* (size* x) 2))))

; Size= 2 or 1
(hashtable-set! concepts "hg23" '(lambda (x) (or* (eqv* (size* x) 2)
                                                  (eqv* (size* x) 1))))


; Size=1 and blue
(hashtable-set! concepts "hg24" '(lambda (x) (and* (eqv* (color* x) 'blue) 
                                                   (eqv* (size* x) 1)) ))

; Size=1 or blue
(hashtable-set! concepts "hg25" '(lambda (x) (or* (eqv* (color* x) 'blue) 
                                                  (eqv* (size* x) 1)) ))

;; One of the largest or one of the smallest (extreme); Easierto learn than size=1or5?
(hashtable-set! concepts "hg26" '(lambda (S) (lambda (x) (or* (contains-SFL-OBJECT* (largest-set* S) x)
                                                              (contains-SFL-OBJECT* (smallest-set* S) x)))))


;; Not one of the largest or one of the smallest (not extreme)
(hashtable-set! concepts "hg27" '(lambda (S) (lambda (x) (and* (not* (contains-SFL-OBJECT* (largest-set* S) x))
                                                               (not* (contains-SFL-OBJECT* (smallest-set* S) x))))))

;; Is one of the largest of its shape
(hashtable-set! concepts "hg28" '(lambda (S) (lambda (x) (contains-SFL-OBJECT* (largest-set* (filter* (lambda (y) (eqv* (shape* y)
                                                                                                                        (shape* x)))
                                                                                                      S))
                                                                               x))))

; The unique largest 
(hashtable-set! concepts "hg29" '(lambda (S) (lambda (x) (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                   (gt* (size* x)
                                                                                        (size* y))))
                                                                  S))))

;; unique largest and blue
(hashtable-set! concepts "hg30" '(lambda (S) (lambda (x) (and* (eqv* (color* x) 'blue)
                                                               (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                         (gt* (size* x)
                                                                                              (size* y))))
                                                                        S)))))

;; unique largest or blue
(hashtable-set! concepts "hg31" '(lambda (S) (lambda (x) (or* (eqv* (color* x) 'blue)
                                                              (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                        (gt* (size* x)
                                                                                             (size* y))))
                                                                       S)))))

; One of the largest
(hashtable-set! concepts "hg32" '(lambda (S) (lambda (x) (forall* (lambda (y) (geq* (size* x)
                                                                                    (size* y)))
                                                                  S))))

;; one of the largest and blue
(hashtable-set! concepts "hg33" '(lambda (S) (lambda (x) (and* (eqv* (color* x) 'blue)
                                                               (forall* (lambda (y) (geq* (size* x)
                                                                                          (size* y)))
                                                                        S)))))

;; one of the largest or blue
(hashtable-set! concepts "hg34" '(lambda (S) (lambda (x) (or* (eqv* (color* x) 'blue)
                                                              (forall* (lambda (y) (geq* (size* x)
                                                                                         (size* y)))
                                                                       S)))))

; There exists a smaller one
(hashtable-set! concepts "hg35" '(lambda (S) (lambda (x) (exists* (lambda (y) (gt* (size* x) (size* y)))
                                                                  S))))

; There exists a smaller blue one
(hashtable-set! concepts "hg36" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (gt* (size* x) (size* y))
                                                                                    (eqv* (color* y) 'blue)))
                                                                  S))))

;Same shape as a blue object
(hashtable-set! concepts "hg37" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                          (shape* x))
                                                                                    (eqv* (color* y) 'blue)))
                                                                  S))))

; Same shape as a blue or circle object
(hashtable-set! concepts "hg38" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                          (shape* x))
                                                                                    (or* (eqv* (color* y) 'blue)
                                                                                         (eqv* (shape* y) 'circle))))
                                                                  S))))

; Same shape as a blue or green object
(hashtable-set! concepts "hg39" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                          (shape* x))
                                                                                    (or* (eqv* (color* y) 'blue)
                                                                                         (eqv* (color* y) 'green))))
                                                                  S))))

;; Same shape as a blue object and not blue
(hashtable-set! concepts "hg40" '(lambda (S) (lambda (x) (and* (not* (eqv* (color* x) 'blue))
                                                               (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                                (shape* x))
                                                                                          (eqv* (color* y) 'blue)))
                                                                        S)))))
                
;; [Same shape as a blue object] or green
(hashtable-set! concepts "hg41" '(lambda (S) (lambda (x)  (or* (eqv* (color* x) 'green)
                                                               (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                                (shape* x))
                                                                                          (eqv* (color* y) 'blue)))
                                                                        S)))))

;; [Same shape as a blue object] and green
(hashtable-set! concepts "hg42" '(lambda (S) (lambda (x)  (and* (eqv* (color* x) 'green)
                                                                (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                                 (shape* x))
                                                                                           (eqv* (color* y) 'blue)))
                                                                         S)))))

;; [Same shape as a blue object] and circle (check selective attention across domains)
(hashtable-set! concepts "hg43" '(lambda (S) (lambda (x)  (and* (eqv* (shape* x) 'circle)
                                                                (exists* (lambda (y) (and* (eqv* (shape* y)
                                                                                                 (shape* x))
                                                                                           (eqv* (color* y) 'blue)))
                                                                         S)))))

;Same shape as the unique largest
(hashtable-set! concepts "hg44" '(lambda (S) (lambda (x) (eqv* (shape* x) (shape* (unique-largest* S))))))


;; Same shape as one of the largest
(hashtable-set! concepts "hg45" '(lambda (S) (lambda (x) (exists* (lambda (y) (eqv* (shape* y)
                                                                                    (shape* x)))
                                                                  (largest-set* S)
                                                                  ))))

;; same shape as one of the largest and blue
(hashtable-set! concepts "hg46" '(lambda (S) (lambda (x) (and* (eqv* (color* x) 'blue)
                                                               (exists* (lambda (y) (eqv* (shape* y)
                                                                                          (shape* x)))
                                                                        (largest-set* S)
                                                                        )))))

;; same shape as one of the largest or blue
(hashtable-set! concepts "hg47" '(lambda (S) (lambda (x) (or* (eqv* (color* x) 'blue)
                                                              (exists* (lambda (y) (eqv* (shape* y)
                                                                                         (shape* x)))
                                                                       (largest-set* S)
                                                                       )))))

;Same shape as the unique largest but not the largest
(hashtable-set! concepts "hg48" '(lambda (S) (lambda (x) (and* (not* (eqv-SFL-object* x (unique-largest* S)))
                                                               (eqv* (shape* x) (shape* (unique-largest* S)))))))


;; Same shape as one of the largest but not one of the largest
(hashtable-set! concepts "hg49" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                    (eqv* (shape* y) (shape* x))))
                                                                  (largest-set* S)
                                                                  ))))

;; unique blue object
(hashtable-set! concepts "hg50" '(lambda (S) (lambda (x) (and* (eqv* (color* x) 'blue)
                                                               (forall* (lambda (y)
                                                                          (or* (eqv-SFL-object* y x)
                                                                               (not* (eqv* (color* y) 'blue))))
                                                                        S)))))

;; unique circle
(hashtable-set! concepts "hg51" '(lambda (S) (lambda (x) (and* (eqv* (shape* x) 'circle)
                                                               (forall* (lambda (y)
                                                                          (or* (eqv-SFL-object* y x)
                                                                               (not* (eqv* (shape* y) 'circle))))
                                                                        S)))))

;; the unique element and is [blue or green]
;; NOTE: THIS ONE WAS MESSED UP TOO -- SHOULD BE Y INSIDE THE Y LAMBDA
(hashtable-set! concepts "hg52" '(lambda (S) (lambda (x) (and* (or* (eqv* (color* x) 'blue)
                                                                    (eqv* (color* x) 'green))
                                                               (forall* (lambda (y)
                                                                          (or* (eqv-SFL-object* y x)
                                                                               (not* (or* (eqv* (color* x) 'blue)
                                                                                          (eqv* (color* x) 'green)))))
                                                                        S)))))

;; the unique element and is [blue and circle] 
;; NOTE: THIS ONE WAS MESSED UP TOO -- SHOULD BE Y INSIDE THE Y LAMBDA
(hashtable-set! concepts "hg53" '(lambda (S) (lambda (x) (and* (and* (eqv* (color* x) 'blue)
                                                                     (eqv* (shape* x) 'circle))
                                                               (forall* (lambda (y)
                                                                          (or* (eqv-SFL-object* y x)
                                                                               (not* (and* (eqv* (color* x) 'blue)
                                                                                           (eqv* (shape* x) 'circle)))))
                                                                        S)))))

;; the unique element and is [blue or circle]
;; NOTE: THIS ONE WAS MESSED UP TOO -- SHOULD BE Y INSIDE THE Y LAMBDA
(hashtable-set! concepts "hg54" '(lambda (S) (lambda (x) (and* (or* (eqv* (color* x) 'blue)
                                                                    (eqv* (shape* x) 'circle))
                                                               (forall* (lambda (y)
                                                                          (or* (eqv-SFL-object* y x)
                                                                               (not* (or* (eqv* (color* x) 'blue)
                                                                                          (eqv* (shape* x) 'circle)))))
                                                                        S)))))

;; The unique largest blue one
(hashtable-set! concepts "hg55" '(lambda (S) (lambda (x) (eqv-SFL-object* x (unique-largest* (filter* (lambda (y) (eqv* (color* y) 'blue))
                                                                                                      S))))))

;; The unique largest blue or green object
(hashtable-set! concepts "hg56" '(lambda (S) (lambda (x) (eqv-SFL-object* x (unique-largest* (filter* (lambda (y) (or* (eqv* (color* y) 'blue)
                                                                                                                       (eqv* (color* y) 'green)))
                                                                                                      S))))))

;; Shape of the unique largest blue one
(hashtable-set! concepts "hg57" '(lambda (S) (lambda (x) (eqv* (shape* x)
                                                               (shape* (unique-largest* (filter* (lambda (y) (eqv* (color* y) 'blue))
                                                                                                 S)))))))

;; Same shape as one of the largest blue ones
(hashtable-set! concepts "hg58" '(lambda (S) (lambda (x) (exists* (lambda (y) (eqv* (shape* y)
                                                                                    (shape* x)))
                                                                  (largest-set* (filter* (lambda (y) (eqv* (color* y) 'blue))
                                                                                         S))
                                                                  ))))


;; There exists another object with the same shape
(hashtable-set! concepts "hg59" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                    (eqv* (shape* y) (shape* x))))
                                                                  S
                                                                  ))))

;; There exists another object with the same size
(hashtable-set! concepts "hg60" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                    (eqv* (size* y) (size* x))))
                                                                  S
                                                                  ))))


;; [There exists another object with the same shape] or blue
(hashtable-set! concepts "hg61" '(lambda (S) (lambda (x) (or* (eqv* (color* x) 'blue)
                                                              (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                         (eqv* (shape* y) (shape* x))))
                                                                       S
                                                                       )))))

;; [there exists antoher object with the same shape] and blue
(hashtable-set! concepts "hg62" '(lambda (S) (lambda (x) (and* (eqv* (color* x) 'blue)
                                                               (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                          (eqv* (shape* y) (shape* x))))
                                                                        S
                                                                        )))))

;; There exists another object with the same color
(hashtable-set! concepts "hg63" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                    (eqv* (color* y) (color* x))))
                                                                  S 
                                                                  ))))

;; There does not exist another object with the same shape (is the sole instance of a shape)
(hashtable-set! concepts "hg64" '(lambda (S) (lambda (x) (not* (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                          (eqv* (shape* y) (shape* x))))
                                                                        S)))))

;; There exists another object with the same shape and color
(hashtable-set! concepts "hg65" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* (eqv-SFL-object* x y))
                                                                                    (and* (eqv* (shape* y) (shape* x))
                                                                                          (eqv* (color* y) (color* x)))))
                                                                  S))))

;; Every other object with the same shape is your color
(hashtable-set! concepts "hg66" '(lambda (S) (lambda (x) (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                   (eqv* (color* y)
                                                                                         (color* x))))
                                                                  (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                           S)
                                                                  ))))

;; [Every other object with the same shape is your color] or blue
(hashtable-set! concepts "hg67" '(lambda (S) (lambda (x) (or* (eqv* (color* x) 'blue)
                                                              (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                        (eqv* (color* y)
                                                                                              (color* x))))
                                                                       (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                                S)
                                                                       )))))

;; [Every other object with the same shape is your color] or circle
(hashtable-set! concepts "hg68" '(lambda (S) (lambda (x) (or* (eqv* (shape* x) 'circle)
                                                              (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                        (eqv* (color* y)
                                                                                              (color* x))))
                                                                       (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                                S)
                                                                       )))))


;; Every object with your shape is blue
;; THIS WAS A MISTAKE: SHOULD BE (eqv* (color* y ....
(hashtable-set! concepts "hg69" '(lambda (S) (lambda (x) (forall* (lambda (y) (eqv* (color* x) 'blue))
                                                                  (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                           S)
                                                                  ))))


;; Every other object with the same shape is your color
(hashtable-set! concepts "hg70" '(lambda (S) (lambda (x) (forall* (lambda (y) (or* (eqv-SFL-object* x y)
                                                                                   (eqv* (color* y)
                                                                                         (color* x))))
                                                                  (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                           S)
                                                                  ))))

;; Every other object with the same shape is not your color
(hashtable-set! concepts "hg71" '(lambda (S) (lambda (x) (forall* (lambda (y) (or* ( eqv-SFL-object* x y)
                                                                                   (not* (eqv* (color* y)
                                                                                               (color* x)))))
                                                                  (filter* (lambda (y) (eqv* (shape* y) (shape* x)))
                                                                           S)
                                                                  ))))

;; There exists a blue object with the same shape
(hashtable-set! concepts "hg72" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (not* ( eqv-SFL-object* x y))
                                                                                    (and* (eqv* (shape* y) (shape* x))
                                                                                          (eqv* (color* y) 'blue))))
                                                                  S))))

;; There exists one object with the same shape, and one with the same color
(hashtable-set! concepts "hg73" '(lambda (S) (lambda (x) (and* (exists* (lambda (y) (and* (not* ( eqv-SFL-object* x y))
                                                                                          (eqv* (color* y) (color* x))))
                                                                        S 
                                                                        )
                                                               (exists* (lambda (y) (and* (not* ( eqv-SFL-object* x y))
                                                                                          (eqv* (shape* y) (shape* x))))
                                                                        S) ))))

;; There exists one object with the same shape, and a different one with the same color
(hashtable-set! concepts "hg74" '(lambda (S) (lambda (x) (exists* (lambda (y) 
                                                                    (exists* (lambda (z) (and* (not* ( eqv-SFL-object* x y))
                                                                                               (and* (not* ( eqv-SFL-object* x z))
                                                                                                     (and* (not* ( eqv-SFL-object* y z))
                                                                                                           (and* (eqv* (shape* y) (shape* x))
                                                                                                                 (eqv* (color* z) (color* x)))))))
                                                                             S))
                                                                  S))))

;; There exists one object with the same shape which has another object of the same color
(hashtable-set! concepts "hg75" '(lambda (S) (lambda (x) (exists* (lambda (y) 
                                                                    (exists* (lambda (z) (and* (not* (eqv-SFL-object* x y))
                                                                                               (and* (not* (eqv-SFL-object* x z))
                                                                                                     (and* (not* (eqv-SFL-object* y z))
                                                                                                           (and* (eqv* (shape* y) (shape* x))
                                                                                                                 (eqv* (color* y) (color* z)))))))
                                                                             S))
                                                                  S))))

;; Shares a feature with every other object
(hashtable-set! concepts "hg76" '(lambda (S) (lambda (x) (forall* (lambda (y) (or* (eqv* (shape* x) (shape* y))
                                                                                   (or* (eqv* (color* x) (color* y))
                                                                                        (eqv* (size* x) (size* y)))))
                                                                  S
                                                                  ))))


;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; Additional concepts added Oct 26 2010
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;; Circle implies blue
(hashtable-set! concepts "hg77" '(lambda (x) (implies* 
                                              (eqv* (shape* x) 'circle)
                                              (eqv* (color* x) 'blue))) )

;; Blue implies circle
(hashtable-set! concepts "hg78" '(lambda (x) (implies* 
                                              (eqv* (color* x) 'blue)
                                              (eqv* (shape* x) 'circle))) )

;; [not blue] implies circle
(hashtable-set! concepts "hg79" '(lambda (x) (implies* 
                                              (not* (eqv* (color* x) 'blue))
                                              (eqv* (shape* x) 'circle))) )

;; [not blue] implies [not circle]
(hashtable-set! concepts "hg80" '(lambda (x) (implies* 
                                              (not* (eqv* (color* x) 'blue))
                                              (not* (eqv* (shape* x) 'circle)))))


;; blue implies size=1
(hashtable-set! concepts "hg81" '(lambda (x) (implies* 
                                              (eqv* (color* x) 'blue)
                                              (eqv* (size* x) 1))))

;; [circle or triangle] implies blue
(hashtable-set! concepts "hg82" '(lambda (x) (implies* 
                                              (or* (eqv* (shape* x) 'circle) (eqv* (shape* x) 'triangle))
                                              (eqv* (color* x) 'blue))) )

;; [circle and blue] or [triangle and green]
(hashtable-set! concepts "hg83" '(lambda (x) (or* 
                                              (and* (eqv* (shape* x) 'circle) (eqv* (color* x) 'blue))
                                              (and* (eqv* (shape* x) 'triangle) (eqv* (color* x) 'green)))))

;; [circle or blue] or [triangle and green]
(hashtable-set! concepts "hg84" '(lambda (x) (or* 
                                              (or* (eqv* (shape* x) 'circle) (eqv* (color* x) 'blue))
                                              (and* (eqv* (shape* x) 'triangle) (eqv* (color* x) 'green)))))
;; circle or [blue and triangle]
(hashtable-set! concepts "hg85" '(lambda (x) (or* 
                                              (eqv* (shape* x) 'circle)
                                              (and* (eqv* (shape* x) 'triangle) (eqv* (color* x) 'blue)))))

;; circle or [blue implies triangle]
(hashtable-set! concepts "hg86" '(lambda (x) (or* 
                                              (eqv* (shape* x) 'circle)
                                              (implies* (eqv* (shape* x) 'triangle) (eqv* (color* x) 'blue)))))


;; There exists a blue object of the same shape (may be this object)
(hashtable-set! concepts "hg87" '(lambda (S) (lambda (x) (exists* (lambda (y)(and* (eqv* (shape* y) (shape* x))
                                                                                   (eqv* (color* y) 'blue)))
                                                                  S))))

;; Same size as a circle
(hashtable-set! concepts "hg88" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (eqv* (size* y) (size* x))
                                                                                    (eqv* (shape* y) 'circle)))
                                                                  S
                                                                  ))))

;; Same shape as another object which is blue
(hashtable-set! concepts "hg89" '(lambda (S) (lambda (x) (exists-other-SFL* (lambda (y) (and* (eqv* (shape* y) (shape* x))
                                                                                              (eqv* (color* y) 'blue)))
                                                                            x
                                                                            S
                                                                            ))))


;; Same shape as another [blue or green] object
(hashtable-set! concepts "hg90" '(lambda (S) (lambda (x) (exists-other-SFL* (lambda (y) (and* (eqv* (shape* y) (shape* x))
                                                                                              (or* (eqv* (color* y) 'blue)
                                                                                                   (eqv* (color* y) 'green))))
                                                                            x
                                                                            S
                                                                            ))))

;;;; Same shape as a [blue or green] object (potentially itself)
(hashtable-set! concepts "hg91" '(lambda (S) (lambda (x) (exists* (lambda (y) (and* (eqv* (shape* y) (shape* x))
                                                                                    (or* (eqv* (color* y) 'blue)
                                                                                         (eqv* (color* y) 'green))))
                                                                  S
                                                                  ))))

;; The unique object that is blue
(hashtable-set! concepts "hg92" '(lambda (S) (lambda (x) (is-the-unique* x
                                                                         (lambda (y) (eqv* (color* y) 'blue))
                                                                         S))))

;; The unique object that is [blue or circle]
(hashtable-set! concepts "hg93" '(lambda (S) (lambda (x) (is-the-unique* x
                                                                         (lambda (y) (or* (eqv* (color* y) 'blue)
                                                                                          (eqv* (shape* y) 'circle)))
                                                                         S))))

;; The unique object that is [blue or green]
(hashtable-set! concepts "hg94" '(lambda (S) (lambda (x) (is-the-unique* x
                                                                         (lambda (y) (or* (eqv* (color* y) 'blue)
                                                                                          (eqv* (color* y) 'green)))
                                                                         S))))

;; The unique object that is [blue and circle]
(hashtable-set! concepts "hg95" '(lambda (S) (lambda (x) (is-the-unique* x
                                                                         (lambda (y) (and* (eqv* (color* y) 'blue)
                                                                                           (eqv* (shape* y) 'circle)))
                                                                         S))))

;; The unique object
(hashtable-set! concepts "hg96" '(lambda (S) (lambda (x) (eqv-SFL-object* (the-unique* (lambda (y) #t)
                                                                                       S)
                                                                          x))))

;; Same size as the unique blue object
(hashtable-set! concepts "hg97" '(lambda (S) (lambda (x) (eqv-shape* x (the-unique* (lambda (y) (eqv* (color* y) 'blue))
                                                                                    S)))))

;; unique smallest
(hashtable-set! concepts "hg98" '(lambda (S) (lambda (x) (eqv-SFL-object* x (unique-smallest* S)))))

;; one of the smallest
(hashtable-set! concepts "hg99" '(lambda (S) (lambda (x) (contains-SFL-OBJECT* (smallest-set* S) x))))


;; one of the smallest of its shape
(hashtable-set! concepts "hg100" '(lambda (S) (lambda (x) (contains-SFL-OBJECT* (smallest-set* (filter* (lambda (y) (eqv* (shape* y)
                                                                                                                          (shape* x)))
                                                                                                        S))
                                                                                x))))

;; the unique smallest of its shape
(hashtable-set! concepts "hg101" '(lambda (S) (lambda (x) (eqv-SFL-object* (unique-smallest* (filter* (lambda (y) (eqv* (shape* y)
                                                                                                                        (shape* x)))
                                                                                                      S))
                                                                           x))))

; Exactly one other element is blue
(hashtable-set! concepts "hg102" '(lambda (S) (lambda (x) (exists-exactly-one-other* (lambda (y) (eqv* (color* y) 'blue))
                                                                                     x
                                                                                     S))))

; Exactly one element is blue
(hashtable-set! concepts "hg103" '(lambda (S) (lambda (x) (exists-exactly-one* (lambda (y) (eqv* (color* y) 'blue))
                                                                               S))))

;; Exactly one other element is the same color
(hashtable-set! concepts "hg104" '(lambda (S) (lambda (x) (exists-exactly-one-other* (lambda (y) (eqv* (color* y) (color* x)))
                                                                                   x
                                                                                   S))))

;; Same shape as exactly one blue object
(hashtable-set! concepts "hg105" '(lambda (S) (lambda (x) (exists-exactly-one* (lambda (y) (eqv-shape* y x))
                                                                              (filter blue?* S)))))

;; Same shape as exactly one other blue object
(hashtable-set! concepts "hg106" '(lambda (S) (lambda (x) (exists-exactly-one-other* (lambda (y) (eqv-shape* y x))
                                                                                    x
                                                                                    (filter blue?* S)))))

;; Same shape as exactly one blue object
(hashtable-set! concepts "hg107" '(lambda (S) (lambda (x) (exists-exactly-one* (lambda (y) (eqv-shape* y x))
                                                                              (filter blue?* S)))))

;; Every other object with the same shape is blue
(hashtable-set! concepts "hg108" '(lambda (S) (lambda (x) (forall-other-SFL* blue?*
                                                                        x
                                                                        (filter (lambda (y) (eqv-shape* y x)) S)))))

;; Every object with the same shape is blue
(hashtable-set! concepts "hg109" '(lambda (S) (lambda (x) (forall* blue?*
                                                                  (filter (lambda (y) (eqv-shape* y x)) S)))))


;; Every-other-atleastone object with the same shape is blue
(hashtable-set! concepts "hg110" '(lambda (S) (lambda (x) (forall-other-SFL-atleastone* blue?*
                                                                                        x
                                                                                        (filter (lambda (y) (eqv-shape* y x)) S)))))

;; Every-atleastone object with the same shape is the same color
(hashtable-set! concepts "hg111" '(lambda (S) (lambda (x) (forall-atleastone* (lambda (y) (eqv-color* y x))
                                                                              (filter (lambda (y) (eqv-shape* y x)) S)))))

;; Every-other-atleastone object with the same shape is not the same color
(hashtable-set! concepts "hg112" '(lambda (S) (lambda (x) (forall-other-SFL-atleastone* (lambda (y) (not* (eqv-color* y x)))
                                                                                        x
                                                                                        (filter (lambda (y) (eqv-shape* y x)) S)))))




;;;;;;;;;;;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;; All elements are the same color (whatever it is)
;(hashtable-set! concepts "hg113" '(lambda (S) (lambda (x) (exists-color* (lambda (p) (forall* (lambda (y) (p y)) S))))))
;                                   
;; All elements are the same shape (whatever it is)
;(hashtable-set! concepts "hg114" '(lambda (S) (lambda (x) (exists-shape* (lambda (p) (forall* (lambda (y) (p y)) S))))))
;
;;; There is a color such that no object is that color
;(hashtable-set! concepts "hg115" '(lambda (S) (lambda (x) (exists-color* (lambda (p) (exists* (lambda (y) (not (p y)) S)))))))
;
;;; There is a shape such that no object is that shape
;(hashtable-set! concepts "hg116" '(lambda (S) (lambda (x) (exists-shape* (lambda (p) (exists* (lambda (y) (not (p y))) S))))))
;
;;; There is a color such that no object other than x is that color
;(hashtable-set! concepts "hg117" '(lambda (S) (lambda (x) (exists-shape* (lambda (p) (exists* (lambda (y) (not (p y))) (non-xes x S)))))))
;
;;; There is a shape such that no object other than x is that shape
;(hashtable-set! concepts "hg118" '(lambda (S) (lambda (x) (exists-shape* (lambda (p) (exists* (lambda (y) (not (p y))) (non-xes x S)))))))
;
;;; exists exactly 1 blue
;(hashtable-set! concepts "hg119" '(lambda (S) (lambda (x) (exists-exactly-one* blue?* S))))
;
;;; exists exactly 1 circle
;(hashtable-set! concepts "hg120" '(lambda (S) (lambda (x) (exists-exactly-one* circle?* S))))
;
;;; exists 0 or 1 blue
;(hashtable-set! concepts "hg121" '(lambda (S) (lambda (x) (exists-one-or-fewer* blue?* S))))
;
;;; exists 0 or 1 circle
;(hashtable-set! concepts "hg122" '(lambda (S) (lambda (x) (exists-one-or-fewer* circle?* S))))
;
;;; All elements are blue
;(hashtable-set! concepts "hg123" '(lambda (S) (lambda (x) (forall* blue?* S))))
;
;;; All elements are circles
;(hashtable-set! concepts "hg124" '(lambda (S) (lambda (x) (forall* circle?* S))))
;
;;; exists exactly 1 blue other than x
;(hashtable-set! concepts "hg125" '(lambda (S) (lambda (x) (exists-exactly-one* blue?* (non-xes x S)))))
;
;;; exists exactly 1 circle other than x
;(hashtable-set! concepts "hg126" '(lambda (S) (lambda (x) (exists-exactly-one* circle?* (non-xes x S)))))
;
;;; exists 0 or 1 blue other than x
;(hashtable-set! concepts "hg127" '(lambda (S) (lambda (x) (exists-one-or-fewer* blue?* (non-xes x S)))))
;
;;; exists 0 or 1 circle other than x
;(hashtable-set! concepts "hg128" '(lambda (S) (lambda (x) (exists-one-or-fewer* circle?* (non-xes x S)))))
;
;;; exists exactly 1 other object of the same color as x
;(hashtable-set! concepts "hg129" '(lambda (S) (lambda (x) (exists-exactly-one* (lambda (y) (equal-color* y x))
;                                                                                (non-xes x S)))))
;
;;; exists exactly 1 other object of the same shape as x
;(hashtable-set! concepts "hg130" '(lambda (S) (lambda (x) (exists-exactly-one* (lambda (y) (equal-shape* y x))
;                                                                                (non-xes x S)))))
;
;;; exists 0 or 1 objects of the same color as x
;(hashtable-set! concepts "hg131" '(lambda (S) (lambda (x) (exists-one-or-fewer* (lambda (y) (equal-color* y x))
;                                                                                (non-xes x S)))))
;
;;; exists 0 or 1 objects of the same shape as x
;(hashtable-set! concepts "hg132" '(lambda (S) (lambda (x) (exists-one-or-fewer* (lambda (y) (equal-shape* y x))
;                                                                                (non-xes x S)))))
;
;;; All elements are blue other than x
;(hashtable-set! concepts "hg133" '(lambda (S) (lambda (x) (forall* blue?* (non-xes x S)))))
;
;;; All elements are circles other than x
;(hashtable-set! concepts "hg134" '(lambda (S) (lambda (x) (forall* circle?* (non-xes x S)))))
;
;;; More (>) other objects of the same shape than the same color
;(hashtable-set! concepts "hg135" '(lambda (S) (lambda (x) (satisfiers>* (lambda (y) (same-shape* y x))
;                                                                        (lambda (y) (same-color* y x))
;                                                                        (non-xes x S)))))
;
;;; More (>=) other objects of the same shape than the same color
;(hashtable-set! concepts "hg136" '(lambda (S) (lambda (x) (satisfiers>=* (lambda (y) (same-shape* y x))
;                                                                         (lambda (y) (same-color* y x))
;                                                                         (non-xes x S)))))
;
;;; as many other objects with the same shape as the same color
;(hashtable-set! concepts "hg137" '(lambda (S) (lambda (x) (satisfiers=* (lambda (y) (same-shape* y x))
;                                                                        (lambda (y) (same-color* y x))
;                                                                        (non-xes x S)))))

