(define exists-shapes (lambda (p) (exists (lambda (f) (p f))
                                          (list circle?* triangle?* rectangle?*))))

(define exists-colors (lambda (p) (exists (lambda (f) (p f))
                                          (list blue?* green?* yellow?*))))

(define exists-sizes (lambda (p) (exists (lambda (f) (p f))
                                         (list size1?* size2?* size3?*))))

(define (non-xes x S) ;; get the 
  (set-difference-SFL* S (list x)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;; Grammars 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
(define          Language-OnlyFeatures (make-PCFG))
(Language-OnlyFeatures 'add-rules!  
                       
                       '(FUNCTION (lambda (S) (lambda (x) BOOL)) 1) ; simpler version that always does S
                       
                       ; Basic predicates
                       '(OBJ->BOOL COLOR 1)
                       '(COLOR blue?* 1)
                       '(COLOR green?* 1)
                       '(COLOR yellow?* 1)
                       
                       '(OBJ->BOOL SHAPE 1)
                       '(SHAPE circle?* 1)
                       '(SHAPE rectangle?* 1)
                       '(SHAPE triangle?* 1)
                       
                       '(OBJ->BOOL SIZE 1)
                       '(SIZE size3?* 1)
                       '(SIZE size2?* 1)
                       '(SIZE size1?* 1)
                       
                       '(BOOL (OBJ->BOOL x) 10) ;; if you arent conditioning on sets, you cant reference any interesting objects other than x
                       )
;


(define          Language-SimpleBoolean (make-PCFG))
(Language-SimpleBoolean 'add-rules!  
                        
                        '(FUNCTION (lambda (S) (lambda (x) BOOL)) 1) ; simpler version that always does S
                        '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
                        '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
                        
                        ; Basic predicates
                        '(OBJ->BOOL COLOR 1)
                        '(COLOR blue?* 1)
                        '(COLOR green?* 1)
                        '(COLOR yellow?* 1)
                        
                        '(OBJ->BOOL SHAPE 1)
                        '(SHAPE circle?* 1)
                        '(SHAPE rectangle?* 1)
                        '(SHAPE triangle?* 1)
                        
                        '(OBJ->BOOL SIZE 1)
                        '(SIZE size3?* 1)
                        '(SIZE size2?* 1)
                        '(SIZE size1?* 1)
                        
                        '(BOOL (OBJ->BOOL SET-OBJ) 10) ;; if you arent conditioning on sets, you cant reference any interesting objects other than x
                        '(SET-OBJ x 1) ;; just to have this here
                        
                        '(BOOL (and* BOOL BOOL) 1)
                        '(BOOL (not* BOOL) 1)
                        '(BOOL (or* BOOL BOOL) 1)
                        ; '(BOOL #t 1)
                        ; '(BOOL #f 1)
                        
                        
                        )
;
;
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;
(define          Language-NAND (make-PCFG))
(Language-NAND 'add-rules!  
               
               '(FUNCTION (lambda (S) (lambda (x) BOOL)) 1) ; simpler version that always does S
               '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
               '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
               
               ; Basic predicates
               '(OBJ->BOOL COLOR 1)
               '(COLOR blue?* 1)
               '(COLOR green?* 1)
               '(COLOR yellow?* 1)
               
               '(OBJ->BOOL SHAPE 1)
               '(SHAPE circle?* 1)
               '(SHAPE rectangle?* 1)
               '(SHAPE triangle?* 1)
               
               '(OBJ->BOOL SIZE 1)
               '(SIZE size3?* 1)
               '(SIZE size2?* 1)
               '(SIZE size1?* 1)
               
               '(BOOL (OBJ->BOOL x) 10)
               
               '(BOOL (nand* BOOL BOOL) 1)
               '(BOOL #t 1)
               '(BOOL #f 1)
               )


(define          Language-NOR (make-PCFG))
(Language-NOR 'add-rules!  
              
              '(FUNCTION (lambda (S) (lambda (x) BOOL)) 1) ; simpler version that always does S
              '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
              '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
              
              ; Basic predicates
              '(OBJ->BOOL COLOR 1)
              '(COLOR blue?* 1)
              '(COLOR green?* 1)
              '(COLOR yellow?* 1)
              
              '(OBJ->BOOL SHAPE 1)
              '(SHAPE circle?* 1)
              '(SHAPE rectangle?* 1)
              '(SHAPE triangle?* 1)
              
              '(OBJ->BOOL SIZE 1)
              '(SIZE size3?* 1)
              '(SIZE size2?* 1)
              '(SIZE size1?* 1)
              
              '(BOOL (OBJ->BOOL x) 10)
              
              '(BOOL (nor* BOOL BOOL) 1)
              '(BOOL #t 1)
              '(BOOL #f 1)
              )
;
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;; Other languages: DNF, CNF, Horn Clauses
;
(define          Language-CNF (make-PCFG))
(Language-CNF 'add-rules!
              
              '(FUNCTION (lambda (S) (lambda (x) ANDCLAUSE)) 1) ; simpler version that always does S
              '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
              '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
              
              ; Basic predicates
              '(OBJ->BOOL COLOR 1)
              '(COLOR blue?* 1)
              '(COLOR green?* 1)
              '(COLOR yellow?* 1)
              
              '(OBJ->BOOL SHAPE 1)
              '(SHAPE circle?* 1)
              '(SHAPE rectangle?* 1)
              '(SHAPE triangle?* 1)
              
              '(OBJ->BOOL SIZE 1)
              '(SIZE size3?* 1)
              '(SIZE size2?* 1)
              '(SIZE size1?* 1)
              
              '(PRIM (OBJ->BOOL x) 10) 
              '(PRIM (not* (OBJ->BOOL x)) 10)
              
              '(ANDCLAUSE (and* ORCLAUSE ANDCLAUSE) 1)
              '(ANDCLAUSE ORCLAUSE 1)
              
              '(ORCLAUSE (or* PRIM ORCLAUSE) 1)
              '(ORCLAUSE PRIM 1)
              )
;
(define          Language-DNF (make-PCFG))
(Language-DNF 'add-rules!
              
              '(FUNCTION (lambda (S) (lambda (x) ORCLAUSE)) 1) ; simpler version that always does S
              '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
              '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
              
              ; Basic predicates
              '(OBJ->BOOL COLOR 1)
              '(COLOR blue?* 1)
              '(COLOR green?* 1)
              '(COLOR yellow?* 1)
              
              '(OBJ->BOOL SHAPE 1)
              '(SHAPE circle?* 1)
              '(SHAPE rectangle?* 1)
              '(SHAPE triangle?* 1)
              
              '(OBJ->BOOL SIZE 1)
              '(SIZE size3?* 1)
              '(SIZE size2?* 1)
              '(SIZE size1?* 1)
              
              '(PRIM (OBJ->BOOL x) 10) 
              '(PRIM (not* (OBJ->BOOL x)) 10)
              
              '(ANDCLAUSE (and* PRIM ANDCLAUSE) 1)
              '(ANDCLAUSE PRIM 1)
              
              '(ORCLAUSE (or* ANDCLAUSE ORCLAUSE) 1)
              '(ORCLAUSE ANDCLAUSE 1)
              )
;




(define          Language-Implies (make-PCFG))
(Language-Implies 'copy-rules Language-SimpleBoolean)
(Language-Implies 'add-rules!
                  '(BOOL (implies* BOOL BOOL) 1)
                  )

(define          Language-Biconditional (make-PCFG))
(Language-Biconditional 'copy-rules Language-SimpleBoolean)
(Language-Biconditional 'add-rules!
                        '(BOOL (iff* BOOL BOOL) 1)
                        )

(define          Language-FullBoolean (make-PCFG))
(Language-FullBoolean 'copy-rules Language-SimpleBoolean)
(Language-FullBoolean 'add-rules!
                      '(BOOL (implies* BOOL BOOL) 1)
                      '(BOOL (iff* BOOL BOOL) 1)
                      )


;; And horn clauses
(define          Language-HornClause (make-PCFG))
(Language-HornClause 'add-rules!
                     
                     '(FUNCTION (lambda (S) (lambda (x) HORN-CONJ)) 1) 
                     '(FUNCTION (lambda (S) (lambda (x) (truethunk))) 1)
                     '(FUNCTION (lambda (S) (lambda (x) (falsethunk))) 1)
                     
                     ; Basic predicates
                     '(OBJ->BOOL COLOR 1)
                     '(COLOR blue?* 1)
                     '(COLOR green?* 1)
                     '(COLOR yellow?* 1)
                     
                     '(OBJ->BOOL SHAPE 1)
                     '(SHAPE circle?* 1)
                     '(SHAPE rectangle?* 1)
                     '(SHAPE triangle?* 1)
                     
                     '(OBJ->BOOL SIZE 1)
                     '(SIZE size3?* 1)
                     '(SIZE size2?* 1)
                     '(SIZE size1?* 1)
                     
                     '(HORN-CONJ HORN-CLAUSE 1)
                     '(HORN-CONJ (and* HORN-CLAUSE HORN-CONJ) 1)
                     
                     '(HORN-CLAUSE (implies* CONJ PRIM) 1)
                     '(HORN-CLAUSE (implies* CONJ #f) 1) ;; added this 2011April4 because it increases expressivity
                     
                     '(PRIM (OBJ->BOOL x) 10) 
                     
                     '(CONJ (and* PRIM CONJ) 1)
                     '(CONJ PRIM 1)
                     
                     )

;
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




; LAM -- lambda abstractions, and  relational operations - same-size, same-shape, same-color, size>, size= 
; QEF -- forall/exists (with S/ (nonxes S) fit in grammar)
; EOQ -- exists-one (exists exactly 1), and exists one or fewer -- fancy 
; CRD -- cardinality operrations (satisfiers>, satisfiers>=, satisfiers=)
; Q2O -- second order operators (exists-shape, exists-color, exists-size)
(define LAM
  '(
    (OBJ->BOOL (*lambda* SET-OBJ BOOL 1) 10) ;; must be able to introduce bound variables
    
    (BOOL (OBJxOBJ->BOOL SET-OBJ SET-OBJ) 10) ; as likely as OBJ->BOOL
    
    (OBJxOBJ->BOOL eqv-SFL-object* 1) ; test if same SFL object
    
    (OBJxOBJ->BOOL size>* 1)
    (OBJxOBJ->BOOL size<=* 1)
    ;  (OBJxOBJ->BOOL size=* 1) ;; is below
    
    (OBJxOBJ->BOOL eqv-size* 1)
    (OBJxOBJ->BOOL eqv-shape* 1)
    (OBJxOBJ->BOOL eqv-color* 1)
    ))
  
(define QEF
  '(    
    
    (BOOL (exists* OBJ->BOOL SET) 1) 
    (BOOL (forall* OBJ->BOOL SET) 1)
    ))

(define CRD
  '(
    (BOOL (satisfiers>* OBJ->BOOL OBJ->BOOL SET) 1) ;; SET here is shared from QEF 
    ; (BOOL (satisfiers>=* OBJ->BOOL OBJ->BOOL SET) 1) ;; lets get rid of this just ot make things  easier
    (BOOL (satisfiers=* OBJ->BOOL OBJ->BOOL SET) 1)
    ))

;; Second order quantifiers
(define Q2O
  '(    
    (BOOL (exists-shapes PRED-LAMBDA) 1)
    (BOOL (exists-colors PRED-LAMBDA) 1)
    (BOOL (exists-sizes PRED-LAMBDA) 1)
    
    (PRED-LAMBDA (*lambda* OBJ->BOOL BOOL 10) 1) ;; introduce a new variable of type OBJ->BOOL, give it weight 10
    ))

;; Exists exactly 1
(define EOQ
  '(    
    ;(BOOL (exists-exactly-one* OBJ->BOOL SET) 1)
    (BOOL (exists-one-or-fewer* OBJ->BOOL SET) 1)
    ))

(define SMC
  '(
    (BOOL (exists-exactly-one* OBJ->BOOL SET) 1) 
    (BOOL (exists-exactly-two* OBJ->BOOL SET) 1) 
    (BOOL (exists-exactly-three* OBJ->BOOL SET) 1) 
    ))

;; NOW DEFUNCT
(define UNQ
  '(
    (BOOL (only-satisfier* SET-OBJ OBJ->BOOL SET) 1) ;; are you the only thing satisfying this predicate?
    ))


;; Here is the language we use as the base for quantifiers
; Note we add lambda
(define          Language-BASE-FOR-QUAN (make-PCFG))
(Language-BASE-FOR-QUAN 'copy-rules Language-FullBoolean)
(Language-BASE-FOR-QUAN 'add-rules!
                        ;; Include these here even though they can't be generated here, because we build off
                        ;; these and need these SETs later, but don't want to introduce them in more than
                        ;; one place later
                        '(SET S 1) ; these are equally likely
                        '(SET (non-xes x S) 1)
                        )
(for l in LAM
  (Language-BASE-FOR-QUAN 'add-rules! l))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Now create every subset of languages
(define all-languages (all-sublists '(QEF EOQ CRD Q2O SMC)))

(for l in all-languages
  (let ((rules (apply append (map (lambda (x) (eval x (interaction-environment)))
                                  l)))
        (sym "Language-LAM"))
    
    (for x in l  ;; Build the symbol for this language (e.g. Language-REQ-Q20, etc)
      (set! sym (string-append sym "-" (symbol->string x))))
    
    ;(edisplay "\"" sym "\", ")    
    
    (eval `(define ,(string->symbol sym) (make-PCFG))
          (interaction-environment))
    
    (eval `(,(string->symbol sym)
            'copy-rules Language-BASE-FOR-QUAN)
          (interaction-environment))
    
    (eval `(apply 
            ,(string->symbol sym)
            (cons (quote add-rules!)
                  (quote ,rules)))
          (interaction-environment))
    ))
(edisplayn "")



;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
