(import (rnrs))

;----------------------------------------------------------------------------------
;; This is a new version which generates hypotheses for concepts
;; by running on some amount of data for a list of concepts
;; This will output several different forms of finite hypothesis spaces:
;
; RAW - all hypotheses
; NONCRAZY - collapsed crazy hypotheses -- CURRENTLY DISABLED
;
;; and will warn if it doesn't find any which perfectly predict the data
;
;;  CHANGES:
;;  - This new version will *not* overwrite old files. 
;;  - This will use multiple TOP-COUNT values
;----------------------------------------------------------------------------------

(load "/home/piantado/Desktop/mit/Libraries/LOT-lib/LOT-lib.scm")
;(define FORCE-LOT-LIB-PATH "/data/scratch/piantado/Libraries/LOT-lib/")
;(load "/data/scratch/piantado/Libraries/LOT-lib/LOT-lib.scm")

; Here we will run a bunch of chains on the data
;; Get a new data point, update  the chains for some large number of steps
;; After updating, your sample is one from each chain
;; The hope is that this sequential method will stop us from
;; having to mix repeatedly since the chains will stay in 
;; relatively good places as samples keep coming. 

(define/log! MCMC-STEPS 10000); 150000) ;; This many steps on *each* amount of dat
(define/log! TOP-COUNT  100) ; how many are in the top N?

(define/log! CONCEPTS '("hg01" "hg02" "hg03" "hg04" "hg05")) 
(define/log! CONCEPT null) ; if this is non-null we only use it
(define/log! LISTS '("L1" "L2")); "L3" "L4"))
(define/log! LIST null)
(define/log! DISPLAY-HYP #f)

;; The amoutn of data you condition on (you predict one more than this!)
(define/log! DATA-AMOUNTS (seq 0 24 1) ) ;; NOTE: For now, this must start at 0 (no data) because of how we compute the mapping of responses to C blocks. Max is 74, not 75. 

(define/log! MCMC-SEARCH-TEMPERATURE 1.0)

(define/log! OUT-PREFIX "mcmc-hypotheses-tmp/") ;; the directory where we put concept-list files with the top hypotheses

(define/log! LANGUAGE "Language-LAM-QEF") ;; Language-Base, Language-Boolean, Language-No-Quantifiers

(define/log! IN-HUMAN-RESPONSES "./Data-Scheme-HELD-OUT.txt") 
(define/log! CONCEPT-DIRECTORY "../concepts/") ;; where the data p<CONCEPT>.L<List>.txt lives
(define/log! QUANTIFIER-FORMAT 0) ; 1 = we are doing quantifiers, else set functions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Set up input and log it
(parse-command-line)
(load "HSFL-General.scm") ;; shared parts of SFL

(define SFL-grammar (eval (string->symbol LANGUAGE) (interaction-environment)))

;; if CONCEPT is defined (e.g. on the command line) then use it, else don't
(set! CONCEPTS (if (not (null? CONCEPT)) (list CONCEPT) CONCEPTS))
(set! LISTS (if (not (null? LIST)) (list LIST)  LISTS))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Randomize the RNG
(random-source-randomize! default-random-source) 

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; Initialize the concept data

;(displayn "Reading in the concept data files.")
(define-concept-data)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Run MH

(for c in CONCEPTS
  (for l in LISTS
    
    (define all-data (hashtable-ref cl2data (list c l) null))
    (define hypotheses (make-PCFG-hashtable)) ;; a list of hypotheses mapped to their best rank
    (define found-perfect #f) ; did we find a perfect hypothesis?
    
    (for di in DATA-AMOUNTS
      
      
      (define D (first-n all-data di ))
      (define initial-H (SFL-grammar 'generate 'FUNCTION MAX-DEPTH))
      
      (define topN (make-TopN TOP-COUNT PCFG-NODE-hash PCFG-NODE-eqv?))
    
      (metropolis-hastings-simple! (make-MH-STATE initial-H null null)
                                   MCMC-STEPS
                                   0
                                   MCMC-SEARCH-TEMPERATURE
                                   (lambda (h) (SFL-grammar 'log-probability h)); ;; a function of hypothesis
                                   compute-likelihood-default   ;; a function of data and hypothesis
                                   D
                                   (lambda (h)  (SFL-grammar 'resample h (+ MAX-DEPTH 2)))
                                   (lambda (mhs) ;; call this on every sample
                                     (if DISPLAY-HYP
                                         (displayn  (MH-STATE-posterior mhs) "\t" (PCFG-delabel (MH-STATE-hypothesis mhs) )))
                                     (topN 'potentially-insert! (MH-STATE-hypothesis mhs) (MH-STATE-posterior mhs)))
                                   PCFG-NODE-eqv?
                                   null ;;a hash to keep track of accept/reject. Can be null
                                   )
      
      ;; Extract the ranked hypotheses and store in topN
      (let ((tns (reverse (topN 'extract-all-sorted!))))
        (map (lambda (h i) ;; go through and if this is a better rank for this hypothesis than we've found, keep it!
               ; make sure this is a node and not some crap from topN
               (if (PCFG-NODE? h) (hashtable-keep-min! hypotheses h i)))
             tns
             (seq 1 (length tns)))))
    
    ;; and output to the file
    ;(map (lambda (x) (edisplayn (PCFG-delabel x))) (vector->list (hashtable-keys hypotheses)))
    (hashtable->file hypotheses (get-hypotheses-file OUT-PREFIX LANGUAGE c l "RAW"))
      
    ))

(exit)


