




;; This takes care of all of the caching etc
;; and implements:

;; (model-push-hypothesis! h resp) -- pushes the hypothesis h with respones resp onto data
;; (model-set-correct! d) -- a set of data
;; (model-set-which-item! w) -- a list saying which item we are on
;; (model-prediction p) -- takes a parameter value and computes the model predicted proportions (possibly with fancy caching)
;; (get-lower-bound) -- retrieve bounds on p
;; (get-upper-bound) 
;; (get-initial-p) -- what p we can start with?


; http://community.schemewiki.org/?simple-object

; -----------------------------------------------------------------------------------------
;; Define across parameters (outside here, so that they are shared)
(define (set-model)
  (set! across-parameters (vector  0.5 0.5 -0.5 -0.5 ))
  (set! across-proposers (vector
                          (lambda (x) (beta-proposal x 200))
                          (lambda (x) (beta-proposal x 200))
                          (lambda (x) (negative-normal-proposal x 0.025))
                          (lambda (x) (negative-normal-proposal x 0.025))))
  (set! make-model make-model:response-biased))

; -----------------------------------------------------------------------------------------
(define (make-model:response-biased)
  
  (define META-description "Uniform over hypotheses." )
  (define META-notes       "(Uses C external library for running-ll)." )
  
  (define correct null) ;; stored in collapsed format
  (define which-item null)
  (define responses null)
  (define responses_cb null)
  (define len 0)
  
  (define (initialize-model) 0)
  (define (get-initial-p) (vector))
  (define (get-proposers) (vector))
  
  (define (model-set-data! d) null)
  
  (define (model-set-correct! c) 
    (set! correct c)
    (set! correct_cb (list->cblock (map to01 correct) 'int))
    (set! len (length correct))
    (set! responses (list (make-list len #f) 
                          (make-list len #t) ))
    (set! responses_cb (list (list->cblock (make-list len 0) 'int)
                             (list->cblock (make-list len 1) 'int)))
    )
  
  (define (model-set-which-item! w) 
    (set! which-item w)
    (set! which-item_cb (list->cblock which-item 'int))
    )
  
  ;; adds h to the model, with c and l
  (define (model-push-hypothesis! h resp)  null)

  (define (clear-hypotheses!) null)
  
  (define (model-prediction p gp ap)
    
  ; (edisplayn len "\t" p "\t" gp "\t" ap "\t" correct "\t" which-item)
  ;  (edisplayn "=-=>" which-item)
    (let* ((PARAM-alpha        (vector-ref ap 0))
           (PARAM-baserate     (vector-ref ap 1))
           (PARAM-llBeta-positive            (vector-ref ap 2))
           (PARAM-llBeta-negative            (vector-ref ap 3))
 
           
           ;; do this so we can use the same below code
           ;; The running LL with the decay
           (wicb (list->cblock which-item 'int)) ;; for some confusing reason, this caching (once) isn't working now but this is fast enough we can recompute
           (running (map (lambda (rscb) (running-ll-C PARAM-alpha  ; (a b llb len resp_cb cor_cb which_cb)
                                                    PARAM-baserate
                                                    PARAM-llBeta-positive
                                                    PARAM-llBeta-negative
                                                    len
                                                    rscb
                                                    correct_cb
                                                    wicb ))
                         responses_cb))
;           ( running (map (lambda (rscb) (running-ll-C PARAM-alpha  ; (a b llb len resp_cb cor_cb which_cb)
;                                                       PARAM-baserate
;                                                       PARAM-llBeta
;                                                       len
;                                                       rscb
;                                                       correct_cb
;                                                       which-item_cb))
;                          responses_cb))
           )
      
      ;(edisplayn "HERE")
      (c_free wicb)
     ; (displaynn len "\t" (length (zip-list running)) "\t" (zip-list running))
      
      (map (lambda (response-i running-i) ;; the responses for the i'th data point and the running-ll for the i'th data point
             (let* ((posterior running-i)
                    (normalizer (logsumexp-lazy posterior)))
               (sum (map (lambda (rij postj) 
                           (* (if (eqv? rij #t) 1.0 0.0) 
                              (exp (- postj normalizer))))
                         response-i
                         posterior))))
           (zip-list responses) ;; these in the right format--first list is all hyps on the first data point
           (zip-list running)
           )))
  
  (define model-entropy (lambda args 0))
  (define model-compute-prior (lambda args 0))
  
  ;; ------------------------------------------------
  ;;; And done, so make this thing a class
  (lambda (message . args)
    (apply (case message 
             ((initialize-model) initialize-model) 
             ((get-initial-p) get-initial-p)
             ((model-push-hypothesis!) model-push-hypothesis!) 
             ((model-set-correct!) model-set-correct!) 
             ((model-set-which-item!) model-set-which-item!) 
             ((model-prediction) model-prediction) 
             ((model-set-data!) model-set-data!)
             ((get-proposers) get-proposers)
	     ((clear-hypotheses!) clear-hypotheses!)
             ((model-entropy) model-entropy)
             ((model-compute-prior) model-compute-prior)
             (else (die (string-append "Unknown Method! " (symbol->string message))))) 
           args)))

                       








;
;(define (make-model)
;  
;  (define description "Simple flat PCFG with no memory decay")
;  
;  (define hypotheses null) ;; a vector of hypotheses
;  (define responses null) ;; a vector of hypotheses
;  (define priors null) ;; a vector of hypotheses
;  (define correct null) ;; stored in collapsed format
;  (define which-item null)
;  
;  (define (model-set-correct! c) (set! correct c))
;  (define (model-set-which-item! w) (set! which-item w))
;  
;  ;; adds h to the model, with c and l
;  (define (model-push-hypothesis! h resp) 
;    (set! hypotheses (cons h hypotheses ))
;    (set! responses  (cons resp responses ))
;    (set! priors     (cons (PCFG-log-probability h) priors))
;    )
;  
;  (define (model-prediction p)
;    (define PARAM-prior-temperature (vector-ref p 0))
;    (define PARAM-alpha             (vector-ref p 1))
;    
;
;    ;; compute the pointwise likelihoods -- one list for each hypothesis
;    ;(define xxx (displayn responses))
;    (define pwl (map (lambda (hresp)
;                       ;(displayn (length hresp) "  " (length correct))
;                       (map (lambda (hri ci)
;                              (if (eqv? hri ci)
;                                  (log PARAM-alpha)
;                                  (log (/ (- 1.0 PARAM-alpha) 2.0))))
;                            hresp
;                            correct))
;                     responses))
;
;     ; the running log likelihood, one for each hypothesis
;    (define running (map (lambda (hpwl)
;                           (running-ll which-item hpwl))
;                    pwl))
;
;   
;    (map (lambda (response-i running-i) ;; the responses for the i'th data point and the running-ll for the i'th data point
;           (let* ((posterior (map + running-i priors) )
;                  (normalizer (logsumexp-lazy posterior)))
;             (apply + (map (lambda (rij postj) 
;                             (* (if (eqv? rij #t) 1 0) 
;                                (exp (- postj normalizer))))
;                           response-i
;                           posterior))))
;         (apply zip responses) ;; these in the right format--first list is all hyps on the first data point
;         (apply zip running)
;         ))
;  
;  ;; ------------------------------------------------
;  ;;; And done, so make this thing a class
;  (lambda (message . args)
;    (apply (case message 
;             ((model-push-hypothesis!) model-push-hypothesis!) 
;             ((model-set-correct!) model-set-correct!) 
;             ((model-set-which-item!) model-set-which-item!) 
;             ((model-prediction) model-prediction) 
;             (else (die "Unknown Method!"))) 
;           args))) 



